const request = require('sync-request');
const logger = require('../helpers/logger');

module.exports = (syncReq) => {
	return async (type, input) => {
		let endpoint = false,
			explorer_url = false,
			returned_field = false;
		switch (type) {
			case 'account':
				endpoint = '/api/accounts?address=' + input;
				break;
			case 'full_account': // DEPRECATED
				explorer_url = 'https://explorer.adamant.im/api/getAccount?address=' + input;
				break;
			case 'delegate_forged':
				endpoint = '/api/delegates/forging/getForgedByAccount?generatorPublicKey=' + input;
				break;
			case 'account_delegates':
				endpoint = '/api/accounts/delegates?address=' + input;
				returned_field = 'delegates';
				break;
			case 'block':
				endpoint = '/api/blocks/get?id=' + input;
				break;
			case 'states':
				endpoint = '/api/states/get';
				if (input) {
					endpoint = endpoint + '?' + input;
				}
				break;
			case 'delegate':
				endpoint = '/api/delegates/get?username=' + input;
				returned_field = 'delegate';
				break;
			case 'delegate_voters':
				endpoint = '/api/delegates/voters?publicKey=' + input;
				returned_field = 'accounts';
				break;
			case 'blocks':
				endpoint = '/api/blocks';
				if (input) {
					endpoint = endpoint + '?' + input;
				}
				returned_field = 'blocks';
				break;
			case 'transaction':
				endpoint = '/api/transactions/get?id=' + input;
				break;
			case 'transactions':
				endpoint = '/api/transactions?' + input.split(' ').join('').split(',').join('&');
				break;
			case 'uri':
				endpoint = '/api/' + input;
				break;
			default:
				logger.error(`ADAMANT endpoint ${type} not implemented yet. Use 'uri' to use not implemented endpoints.`);
				return false;
		}

		try {
			const url = explorer_url || endpoint;
			const res = await syncReq(url, explorer_url);
			if (res && res.success) {
				if (returned_field) {
					return res[returned_field];
				}
				return res;
			}

			logger.warn(`Get request to ADAMANT node was not successful. Type: ${type}, URL: ${url}, Result: ${res && res.error}`);
			return false;

		} catch (e) {
			logger.error(`Failed to process Get request of type ${type} to ADAMANT node. Error: ${e}.`);
			return false;
		}
	};
};
